using System;
using System.Collections;
using System.Configuration;
using System.IO;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Threading;

using Belikov.GenuineChannels;
using Belikov.GenuineChannels.DirectExchange;
using Belikov.GenuineChannels.DotNetRemotingLayer;
using Belikov.GenuineChannels.GenuineTcp;
using Belikov.GenuineChannels.TransportContext;
using Belikov.GenuineChannels.Logbook;
using Belikov.GenuineChannels.Messaging;

using KnownLayer;

namespace Client
{
	/// <summary>
	/// ChatClient demostrates simple client application.
	/// </summary>
	class ChatClient
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			GenuineChunkedStream stream;
			BinaryWriter binaryWriter;

			// wait for the server
			Console.WriteLine("Sleeping for 3 seconds.");
			Thread.Sleep(TimeSpan.FromSeconds(3));

			// setup .NET Remoting
			Console.WriteLine("Configuring network environment...");
			GenuineGlobalEventProvider.GenuineChannelsGlobalEvent += new GenuineChannelsGlobalEventHandler(GenuineChannelsEventHandler);

			Hashtable properties = new Hashtable();
			properties["MaxContentSize"] = "100000000";
			properties["MaxQueuedItems"] = "100000000";
			properties["MaxTotalSize"] = "100000000";

			GenuineTcpChannel genuineTcpChannel = new GenuineTcpChannel(properties, null, null);
			_transportContext = genuineTcpChannel.ITransportContext;
			ChannelServices.RegisterChannel(genuineTcpChannel);

			string objectURI;
			_url = GenuineUtility.Parse(ConfigurationSettings.AppSettings["RemoteHostUri"], out objectURI);

			// -- Initialization -------------------------------------------------------------------
			Random random = new Random();
			int TOTAL_RECORDS = 1000;
			VeryImportantBusinessData[] veryImportant = new VeryImportantBusinessData[TOTAL_RECORDS];
			ImportantBusinessData[] important = new ImportantBusinessData[TOTAL_RECORDS];

			for ( int i = 0; i < TOTAL_RECORDS; i++ )
			{
				veryImportant[i] = new VeryImportantBusinessData();
				important[i] = new ImportantBusinessData();

				veryImportant[i].DT = important[i].DT = DateTime.Now - TimeSpan.FromDays(random.Next(120));
				veryImportant[i].Guid = important[i].Guid = Guid.NewGuid();
				veryImportant[i].I = important[i].I = random.Next(2000000);
				veryImportant[i].L = important[i].L = random.Next(2000000);
				veryImportant[i].Message1 = important[i].Message1 = "Message N" + i.ToString();
				if (random.Next(5) > 2)
				{
					veryImportant[i].AnotherImportantInformation = new VeryImportantBusinessData();
					important[i].ImportantInformation = new ImportantBusinessData();

					veryImportant[i].AnotherImportantInformation.DT = important[i].ImportantInformation.DT = DateTime.Now - TimeSpan.FromDays(random.Next(120));
					veryImportant[i].AnotherImportantInformation.Guid = important[i].ImportantInformation.Guid = Guid.NewGuid();
					veryImportant[i].AnotherImportantInformation.I = important[i].ImportantInformation.I = random.Next(2000000);
					veryImportant[i].AnotherImportantInformation.L = important[i].ImportantInformation.L = random.Next(2000000);
					veryImportant[i].AnotherImportantInformation.Message1 = important[i].ImportantInformation.Message1 = "Message N" + i.ToString();
				}
			}

			// establish the connection
			IDataReceiver iDataReceiver = (IDataReceiver) Activator.GetObject(typeof(IDataReceiver), ConfigurationSettings.AppSettings["RemoteHostUri"] + "/DataReceiver.rem");
			iDataReceiver.DoSomething(null);

			HostInformation remoteHost = _transportContext.KnownHosts.Get(_url);

			const int TRIES = 40;
			DateTime start, finish;

			// -- Measure .NET Remoting ------------------------------------------------------------
			Console.WriteLine("The test using .NET Remoting is starting.");
			start = DateTime.Now;

			for ( int i = 0; i < TRIES; i++ )
				iDataReceiver.DoSomething(veryImportant);

			finish = DateTime.Now;
			Console.WriteLine("Total milliseconds spent: {0}.", (finish - start).TotalMilliseconds);

			// -- Direct Exchange Manager ----------------------------------------------------------
			Console.WriteLine("Direct Exchange Manager with custom unsafe serialization.");
			start = DateTime.Now;

			for ( int i = 0; i < TRIES; i++ )
			{
				// serialize
				stream = new GenuineChunkedStream(false);
				binaryWriter = new BinaryWriter(stream);

				binaryWriter.Write((int) important.Length);
				for ( int o = 0; o < important.Length; o++ )
					important[o].Serialize(stream, binaryWriter);

				// and send
				_transportContext.DirectExchangeManager.SendSync(remoteHost, "/Test/DataReceiver", stream);
			}

			finish = DateTime.Now;
			Console.WriteLine("Total milliseconds spent: {0}.", (finish - start).TotalMilliseconds);

			// -- Measure only serialization -------------------------------------------------------
			Console.WriteLine(".NET Remoting serialization only.");
			start = DateTime.Now;

			for ( int i = 0; i < TRIES; i++ )
				iDataReceiver.DoSomething_OneWay(veryImportant);

			finish = DateTime.Now;
			Console.WriteLine("Total milliseconds spent: {0}.", (finish - start).TotalMilliseconds);

			// -- The last test with Direct Exchange Manager ---------------------------------------
			Console.WriteLine("Async one-way via Direct Exchange Manager.");
			start = DateTime.Now;

			for ( int i = 0; i < TRIES; i++ )
			{
				// serialize once
				stream = new GenuineChunkedStream(false);
				binaryWriter = new BinaryWriter(stream);

				binaryWriter.Write((int) important.Length);
				for ( int o = 0; o < important.Length; o++ )
					important[o].Serialize(stream, binaryWriter);

				// and send the same content several times
				_transportContext.DirectExchangeManager.SendOneWay(remoteHost, "/Test/DataReceiver", (Stream) stream.Clone());
			}

			finish = DateTime.Now;
			Console.WriteLine("Total milliseconds spent: {0}.", (finish - start).TotalMilliseconds);

			Console.ReadLine();
		}

		private static ITransportContext _transportContext;
		private static string _url;

		/// <summary>
		/// Handles network events.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">Event parameters.</param>
		public static void GenuineChannelsEventHandler(object sender, GenuineEventArgs e)
		{
			Console.WriteLine("Global event: {0}\r\nUrl: {1}", e.EventType, e.HostInformation  == null ? "<unknown>" : e.HostInformation.ToString());
		}
	}
}
